"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _buildStandalone = require("./buildStandalone");
var _buildPackage = require("./build-package");
var logger = _interopRequireWildcard(require("../utils/logger"));
var _actionPreflightCheck = _interopRequireDefault(require("../utils/actionPreflightCheck"));
var _getLocation = _interopRequireDefault(require("../utils/getLocation"));
var _selectWorkspaces = require("../utils/selectWorkspaces");
var _setupEnv = require("../utils/setupEnv");
var _getAllWorkspaces = require("../utils/getAllWorkspaces");
var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));
var _execAsync = _interopRequireDefault(require("../utils/execAsync"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
async function build({
  packagePaths,
  preserveModules = true,
  private: includePrivate,
  ancestors,
  descendants,
  changed,
  compareBranch,
  dangerouslyIgnoreCircularDependencies
}) {
  const isSelective = changed || ancestors || descendants || packagePaths.length;
  const modularRoot = (0, _getModularRoot.default)();
  const [allWorkspacePackages] = await (0, _getAllWorkspaces.getAllWorkspaces)(modularRoot);

  // targets are either the set of what's specified in the selective options or all the packages in the monorepo
  const targets = isSelective ? packagePaths : [...allWorkspacePackages.keys()];
  const selectedTargets = await (0, _selectWorkspaces.selectBuildableWorkspaces)({
    targets,
    changed,
    compareBranch,
    descendants,
    ancestors,
    dangerouslyIgnoreCircularDependencies
  });
  if (!selectedTargets.length) {
    logger.log('No workspaces to build');
    process.exit(0);
  }
  logger.debug(`Building the following workspaces in order: ${JSON.stringify(selectedTargets)}`);
  for (const target of selectedTargets) {
    const packageInfo = allWorkspacePackages.get(target);
    try {
      const targetDirectory = await (0, _getLocation.default)(target);
      await (0, _setupEnv.setupEnvForDirectory)(targetDirectory);
      if (packageInfo !== null && packageInfo !== void 0 && packageInfo.modular) {
        // If it's modular, build with Modular
        const targetType = packageInfo.modular.type;
        if (!targetType) throw new Error(`modular.type missing in ${target} package.json`);
        logger.log('\nBuilding', targetType, target);
        if (targetType === 'app' || targetType === 'esm-view') {
          await (0, _buildStandalone.buildStandalone)(target, targetType);
        } else {
          await (0, _buildPackage.buildPackage)(target, preserveModules, includePrivate);
        }
      } else {
        // Otherwise, build by running the workspace's build script
        // We're sure it's here because selectBuildableWorkspaces returns only buildable workspaces.
        logger.log('\nBuilding non-modular package', target);
        await (0, _execAsync.default)(`yarn`, ['workspace', target, 'build'], {
          cwd: modularRoot,
          log: false
        });
      }
    } catch (err) {
      logger.error(`building ${target} failed`);
      throw err;
    }
  }
}
var _default = (0, _actionPreflightCheck.default)(build);
exports.default = _default;
//# sourceMappingURL=index.js.map